classdef amat %< handle  
% CLASSDEF amat:
%   
% -- amat(N,m,n)
%     return a N-by-m-by-n amat object where all its elements are set to 0.
% -- amat(T)
%     return a N-by-m-by-n amat object initialized with the N-by-m-by-n array T.
% -- amat(N,A)
%     return a N-by-m-by-n amat object where all its matrices are initialized
%     with the m-by-n matrix A.  
%
% Copyright (C) 2018 F. Cuvelier
  properties  %(SetAccess=private)  %GetAccess = public
    values=[];  
    N=0;   % number of nr-by-nc matrices
    nr=0;  % number of rows
    nc=0;  % number of columns
    ns=0;  % number of dimensions in the array values: 2 or 3
  end
  
  methods
  
% toto  
    function self=amat(varargin)
    % -- AAA amat(N,m,n)
%     return a N-by-m-by-n amat object where all its elements are set to 0.
% -- amat(T)
%     return a N-by-m-by-n amat object initialized with the N-by-m-by-n array T.
% -- amat(N,A)
%     return a N-by-m-by-n amat object where all its matrices are initialized
%     with the m-by-n matrix A.  

      n=nargin;classname='';
      if ischar(varargin{end}), classname=varargin{end};n=n-1;end
      if n==3  % amat(N,nr,nc)
        self.N=varargin{1};
        self.nr=varargin{2};
        self.nc=varargin{3};
        self.values=zeros(varargin{:});
        self.set_ns() ;
        return
      end
      if n==1  % amat( T )
        if strcmp(class(varargin{1}),'amat')
          self.N=varargin{1}.N ;
          self.nr=varargin{1}.nr;
          self.nc=varargin{1}.nc;
          if isempty(classname),classname=class(varargin{1}.values);end
          if isreal(varargin{1}.values)
            %self.values=zeros(self.N,self.nr,self.nc,class(varargin{1}.values));
            self.values=zeros(self.N,self.nr,self.nc,classname);
          else
            %self.values=complex(zeros(self.N,self.nr,self.nc,class(varargin{1}.values)));
            self.values=complex(zeros(self.N,self.nr,self.nc,classname));
          end
          self.set_ns();
          return
        end
        if isnumeric(varargin{1}) || islogical(varargin{1})
          self.N=size(varargin{1},1);
          self.nr=size(varargin{1},2);
          if isempty(classname) 
            self.values=varargin{1};
          else
            self.values=eval([classname,'(varargin{1})']);
          end
          self.ns=ndims(varargin{1}); % Cannot use set_ns function when uses 'classdef amat' instead of 'classdef amat < handle' 
          %ns=length(size(varargin{1}));
          
          if self.ns==3
            self.nc=size(varargin{1},3);
          elseif self.ns==2
            self.nc=1;
          else
            error('Input parameter must be a matrix or 3D array')
          end
          return
        end
      end
      if n==2  % amat( N, A )  N times a matrix A
        assert( ndims(varargin{2}) == 2)
        if isempty(classname) 
          A=varargin{2};
        else
          A=eval([classname,'(varargin{2})']);
        end
        self.values=permute(repmat(A,[1,1,varargin{1}]),[3,1,2]);
        self.N=varargin{1};
        self.nr=size(varargin{2},1);
        self.nc=size(varargin{2},2);
        self.set_ns();
        return
      end
      error('Input arguments failed!')
    end

    function self=set_ns(self)
      self.ns=length(size(self.values));%ndims(self.values);%length(size(self.values));
    end
    
    function n=dims(a), n=[a.N,a.nr,a.nc];  end
    
    % Arithmetic
    function c=plus(a,b), c=bingenop(a,b,@plus);  end
    function c=uplus(a), c=amat(uplus(a.values));  end
    function c=minus(a,b), c=bingenop(a,b,@minus);  end
    function c=uminus(a), c=amat(uminus(a.values));  end
    function c=times(a,b), c=bingenop(a,b,@times);  end
    function c=rdivide(a,b), c=bingenop(a,b,@rdivide);  end
    function c=ldivide(a,b), c=bingenop(a,b,@ldivide);  end
    function c=power(a,b), c=bingenop(a,b,@power);  end
    %function c=ceil(a), c=amat(ceil(a.values));  end
    %function c=fix(a), c=amat(fix(a.values));  end
    %function c=floor(a), c=amat(floor(a.values));  end
    function c=mod(a,m), c=bingenop(a,m,@mod);  end
    function c=rem(a,b), c=bingenop(a,b,@rem);  end
    %function c=round(a,varargin), c=amat(round(a.values,varargin{:}));  end
    
    % Relational Operations
    function c=eq(a,b), c=bingenop(a,b,@eq);  end
    function c=ge(a,b), c=bingenop(a,b,@ge);  end
    function c=gt(a,b), c=bingenop(a,b,@gt);  end
    function c=le(a,b), c=bingenop(a,b,@le);  end
    function c=lt(a,b), c=bingenop(a,b,@lt);  end
    function c=ne(a,b), c=bingenop(a,b,@ne);  end
    function c=isequal(a,b), c=bingenop(a,b,@isequal);  end
    
    %Logical Operations
    function c=and(a,b), c=bingenop(a,b,@and);  end
    function c=or(a,b), c=bingenop(a,b,@or);  end
    function c=xor(a,b), c=bingenop(a,b,@xor);  end
    function c=not(a), c=amat(not(a.values));  end
    
    % Trigonometry
    function c=sin(a), c=amat(sin(a.values));  end
    function c=sind(a), c=amat(sind(a.values));  end
    function c=asin(a), c=amat(asin(a.values));  end
    function c=asind(a), c=amat(asind(a.values));  end
    function c=sinh(a), c=amat(sinh(a.values));  end
    function c=asinh(a), c=amat(asinh(a.values));  end
    %
    function c=cos(a), c=amat(cos(a.values));  end
    function c=cosd(a), c=amat(cosd(a.values));  end
    function c=acos(a), c=amat(acos(a.values));  end
    function c=acosd(a), c=amat(acosd(a.values));  end
    function c=cosh(a), c=amat(cosh(a.values));  end
    function c=acosh(a), c=amat(acosh(a.values));  end
    % 
    function c=tan(a), c=amat(tan(a.values));  end
    function c=atan(a), c=amat(atan(a.values));  end
    function c=tand(a), c=amat(tand(a.values));  end
    function c=atand(a), c=amat(atand(a.values));  end
    function c=tanh(a), c=amat(tanh(a.values));  end
    function c=atanh(a), c=amat(atanh(a.values));  end
    function c=atan2(a,b), c=bingenop(a,b,@atan2);  end
    function c=atan2d(a,b), c=bingenop(a,b,@atan2d);  end
    % Cosecant functions
    function c=csc(a), c=amat(csc(a.values));  end
    function c=acsc(a), c=amat(acsc(a.values));  end
    function c=cscd(a), c=amat(cscd(a.values));  end
    function c=acscd(a), c=amat(acscd(a.values));  end
    function c=csch(a), c=amat(csch(a.values));  end
    function c=acsch(a), c=amat(acsch(a.values));  end
    % Secant functions
    function c=sec(a), c=amat(sec(a.values));  end
    function c=asec(a), c=amat(asec(a.values));  end
    function c=secd(a), c=amat(secd(a.values));  end
    function c=asecd(a), c=amat(asecd(a.values));  end
    function c=sech(a), c=amat(sech(a.values));  end
    function c=asech(a), c=amat(asech(a.values));  end
    % Cotangent functions
    function c=cot(a), c=amat(cot(a.values));  end
    function c=acot(a), c=amat(acot(a.values));  end
    function c=cotd(a), c=amat(cotd(a.values));  end
    function c=acotd(a), c=amat(acotd(a.values));  end
    function c=coth(a), c=amat(coth(a.values));  end
    function c=acoth(a), c=amat(acoth(a.values));  end
    % other trigonemetric functions
    function c=hypot(a,b), c=bingenop(a,b,@hypot);  end
    function c=deg2rad(a), c=amat(deg2rad(a.values));  end
    function c=rad2deg(a), c=amat(rad2deg(a.values));  end
    
    % Exponents and Logarithms
    function c=exp(a), c=amat(exp(a.values));  end
    function c=expm1(a), c=amat(expm1(a.values));  end
    function c=log(a), c=amat(log(a.values));  end
    function c=reallog(a), c=amat(reallog(a.values));  end
    function c=log1p(a), c=amat(log1p(a.values));  end
    function c=log10(a), c=amat(log10(a.values));  end
    %function c=log2(a), c=amat(log2(a.values));  end
    function c=nextpow2(a), c=amat(nextpow2(a.values));  end
    function c=realpow(a,b), c=bingenop(a,b,@realpow);  end
    function c=sqrt(a), c=amat(sqrt(a.values));  end
    function c=realsqrt(a), c=amat(realsqrt(a.values));  end
    function c=cbrt(a), c=amat(cbrt(a.values));  end
    
    % Complex Numbers
    function c=abs(a), c=amat(abs(a.values));   end
    function c=angle(a), c=amat(angle(a.values));  end
    function c=arg(a), c=amat(arg(a.values));  end
    function c=conj(a), c=amat(conj(a.values));  end
    function c=imag(a), c=amat(imag(a.values));  end
    function c=real(a), c=amat(real(a.values));  end
    function c=sign(a), c=amat(sign(a.values));  end
    
    % Utility Functions
    function c=ceil(a), c=amat(ceil(a.values));  end
    function c=fix(a), c=amat(fix(a.values));  end
    function c=floor(a), c=amat(floor(a.values));  end
    function c=round(a), c=amat(round(a.values));  end
    function c=roundb(a), c=amat(roundb(a.values));  end
    
    function C=int32(A),    C=amat(int32(A.values)); end
    function C=int8(A),     C=amat(int8(A.values)); end
    function C=int16(A),    C=amat(int16(A.values)); end
    function C=int64(A),    C=amat(int64(A.values)); end
    function C=uint32(A),   C=amat(int32(A.values)); end
    function C=uint8(A),    C=amat(uint8(A.values)); end
    function C=uint16(A),   C=amat(uint16(A.values)); end
    function C=uint64(A),   C=amat(uint64(A.values)); end
    %function C=complex(A),  C=amat(complex(A.values)); end
    function C=double(A),   C=amat(double(A.values)); end
    function C=single(A),   C=amat(single(A.values)); end
    function C=logical(A),   C=amat(logical(A.values)); end
    function R=isinteger(A), R=isinteger(A.values);end
    function R=isfloat(A),   R=isfloat(A.values);end
    function R=isreal(A),   R=isreal(A.values);end
    function R=isscalar(A), R=false;end % Needs by Octave otherwize default function return true.
    function R=islogical(A), R=islogical(A.values);end
    function R=isempty(A), R= ( A.N==0 || A.nr==0 || A.nc==0);end
    
    function s=subclass(A), s=class(A.values);end

  end % methods

end % classdef
